(**************************************************************************)
(*                                                                        *)
(*  FORK                                                                  *)
(*                                                                        *)
(*  Author:  Franois Pottier, INRIA Paris-Rocquencourt                   *)
(*  Version: 20101109                                                     *)
(*                                                                        *)
(*  The copyright to this code is held by Institut National de Recherche  *)
(*  en Informatique et en Automatique (INRIA). All rights reserved. This  *)
(*  file is distributed under the license CeCILL-C (see file LICENSE).    *)
(*                                                                        *)
(**************************************************************************)

(* Worlds. *)

(* ------------------------------------------------------------------------- *)

(* A world is a world-parameterized fragment. *)

(* The parameter is used "in depth" -- it is not used, say, to define the
   width of the fragment -- so it is good enough for the parameter to have
   kind [later world] rather than [world]. Of course, this is necessary for
   the kind [world] to be well-formed. *)

kind world = later world -> fragment

(* ------------------------------------------------------------------------- *)

(* Conventions. *)

default kind w : world

default kind x : later world

default kind y : later world

default kind z : later world

(* ------------------------------------------------------------------------- *)

(* Since a world is a parameterized fragment, the empty fragment and the
   operation of fragment concatenation can be lifted to worlds. We obtain
   the empty world and the operation of world concatenation. These are
   analogous to the assertion emp and to the separating conjunction of
   separation logic. *)

(* The empty world. *)

type nil : world =
  \x. fnil

(* World concatenation. *)

type concat : world -> world -> world =
  \w1 w2.
    \x. (w1 x) `@` (w2 x)

(* ------------------------------------------------------------------------- *)

(* Properties. *)

lemma concat_left_unit:
  forall w.
  nil `concat` w = w

lemma concat_right_unit:
  forall w.
  w `concat` nil = w

lemma concat_associative:
  forall w1 w2 w3.
  (w1 `concat` w2) `concat` w3 = w1 `concat` (w2 `concat` w3)

(* ------------------------------------------------------------------------- *)

(* World composition. *)

type o : world -> world -> world =
  \w1 w2.
    (\x. w1 (w2 `o` x)) `concat` w2

(* Properties. *)

lemma nil_compose:
  forall w.
  nil `o` w = w

lemma compose_nil:
  forall w.
  w `o` nil = w

lemma compose_associative:
  forall w1 w2 w3.
  (w1 `o` w2) `o` w3 = w1 `o` (w2 `o` w3)

lemma compose_alternate_definition:
  forall w1 w2.
  w1 `o` w2 = (\x. w1 (w2 `o` x) `@` w2 x)

